/***************************************************************************

	inptport.c

	Input port emulation

***************************************************************************/

#include "ncdz.h"


/***************************************************************************
	Global variables
 **************************************************************************/

int input_map[MAX_INPUTS];
int af_interval = 1;
int neogeo_input_port;

/***************************************************************************
	Private variables
 **************************************************************************/

static const u8 hotkey_mask[11] =
{
//	0xef,	// A
//	0xdf,	// B
//	0xbf,	// C
//	0x7f,	// D
	0xcf,	// A+B
	0xaf,	// A+C
	0x6f,	// A+D
	0x9f,	// B+C
	0x5f,	// B+D
	0x3f,	// C+D
	0x8f,	// A+B+C
	0x4f,	// A+B+D
	0x2f,	// A+C+D
	0x1f,	// B+C+D
	0x0f	// A+B+C+D
};

static const u8 autofire_mask[4] =
{
	0x10,	// A
	0x20,	// B
	0x40,	// C
	0x80,	// D
};

static u8 input_flag[MAX_INPUTS];
static int af_flag[NEOGEO_BUTTON_MAX];
static int af_counter[NEOGEO_BUTTON_MAX];


/*------------------------------------------------------
	Initialize input port emulation
 -----------------------------------------------------*/

void input_init(void)
{
	memset(af_counter, 0, sizeof(af_counter));
	memset(af_flag, 0, sizeof(af_flag));
	memset(input_flag, 0, sizeof(input_flag));
}


/*------------------------------------------------------
	Shutdown input port emulation
 -----------------------------------------------------*/

void input_shutdown(void)
{
}


/*------------------------------------------------------
	Update autofire flag
 -----------------------------------------------------*/

void update_autofire(void)
{
	int i, button;

	button = P1_AF_A;

	for (i = 0; i < 4; i++)
	{
		if (input_flag[button])
		{
			af_counter[i]++;

			if (af_counter[i] >= af_interval)
			{
				af_counter[i] = 0;
				af_flag[i] ^= 1;
			}
		}
		else
		{
			af_counter[i] = 0;
			af_flag[i] = 1;
		}

		button++;
	}
}


/*------------------------------------------------------
	Apply autofire and hotkey
 -----------------------------------------------------*/

static u8 apply_hotkey(u8 value)
{
	int i, button;

	// autofire
	button = P1_AF_A;
	for (i= 0; i < 4; i++)
	{
		if (input_flag[button])
		{
			if (af_flag[i])
				value &= ~autofire_mask[i];
			else
				value |= autofire_mask[i];
		}
		button++;
	}

	// hotkey
	button = P1_AB;
	for (i= 0; i < 11; i++)
	{
		if (input_flag[button]) value &= hotkey_mask[i];
		button++;
	}

	return value;
}


/*------------------------------------------------------
	Read player 1 controler
 -----------------------------------------------------*/

u8 read_inputport1(void)
{
	u8 value = 0xff;

	if (!neogeo_input_port)
	{
		int button;
		u8 mask = 0x01;

		for (button = P1_UP; button <= P1_BUTTOND; button++, mask <<= 1)
			if (input_flag[button]) value &= ~mask;

		value = apply_hotkey(value);
	}

	return value;
}


/*------------------------------------------------------
	Read player 2 controler
 -----------------------------------------------------*/

u8 read_inputport2(void)
{
	u8 value = 0xff;

	if (neogeo_input_port)
	{
		int button;
		u8 mask = 0x01;

		for (button = P1_UP; button <= P1_BUTTOND; button++, mask <<= 1)
			if (input_flag[button]) value &= ~mask;

		value = apply_hotkey(value);
	}

	return value;
}


/*------------------------------------------------------
	Read player 1 & 2 start / select
 -----------------------------------------------------*/

u8 read_inputport3(void)
{
	int button;
	u8 mask, value = 0x0f;

	mask = neogeo_input_port ? 0x04 : 0x01;

	for (button = P1_START; button <= P1_SELECT; button++, mask <<= 1)
		if (input_flag[button]) value &= ~mask;

	return value;
}


/*------------------------------------------------------
	Update input port
 -----------------------------------------------------*/

void update_inputport(void)
{
	int i, buttons = poll_gamepad();

	if ((buttons & PSP_CTRL_RTRIGGER) && (buttons & PSP_CTRL_SELECT))
	{
		buttons &= ~(PSP_CTRL_RTRIGGER | PSP_CTRL_SELECT);
		commandlist(1);
	}

	for (i = 0; i < MAX_INPUTS; i++)
		input_flag[i] = (buttons & input_map[i]) != 0;

	if (input_flag[SHOWMENU]) showmenu();
	if (input_flag[SNAPSHOT]) save_snapshot();
	if (input_flag[SWPLAYER]) neogeo_input_port ^= 1;
}


#ifdef SAVE_STATE

/*------------------------------------------------------
	Save input port state
 -----------------------------------------------------*/

void state_save_inputport(u8 *state_buf)
{
	u8 *in = state_buf;

	state_save_long(&neogeo_input_port, 1);
}


/*------------------------------------------------------
	Load input port state
 -----------------------------------------------------*/

void state_load_inputport(u8 *state_buf)
{
	u8 *out = state_buf;

	state_load_long(&neogeo_input_port, 1);
}

#endif
