/***************************************************************************

	mesbox.c

	Message Box

***************************************************************************/

#include "psp.h"

#define HELP_MES_MAX	13

#define HELP_COLOR_ENABLE	&ui_palette[UI_PAL_SELECT].r, &ui_palette[UI_PAL_SELECT].g, &ui_palette[UI_PAL_SELECT].b
#define HELP_COLOR_DISABLE	&ui_palette[UI_PAL_NORMAL].r, &ui_palette[UI_PAL_NORMAL].g, &ui_palette[UI_PAL_NORMAL].b
#define HELP_COLOR_NORMAL	&ui_palette[UI_PAL_NORMAL].r, &ui_palette[UI_PAL_NORMAL].g, &ui_palette[UI_PAL_NORMAL].b

#define HELP_ENABLE(s1, s2)	{ HELP_COLOR_ENABLE, s1, s2 }
#if JAPANESE_UI
#define HELP_DISABLE(s1)	{ HELP_COLOR_DISABLE, s1, "gp܂" }
#else
#define HELP_DISABLE(s1)	{ HELP_COLOR_DISABLE, s1, "Not use" }
#endif
#define HELP_SKIP			{ HELP_COLOR_NORMAL, "\n", "\0" }
#define HELP_EOM			{ HELP_COLOR_NORMAL, "\0", "\0" }


/***************************************************************************
	Private structures
 ***************************************************************************/

typedef struct message_t
{
	int *r, *g, *b;
	const char *text1;
	const char *text2;
} UI_MESSAGE;

typedef struct help_t
{
	const char *menu_name;
	const UI_MESSAGE mes[HELP_MES_MAX];
} UI_HELP;


const static UI_HELP help_mes[HELP_NUM_MAX] =
{
	// HELP_FILEBROWSER
	{
#if JAPANESE_UI
		"t@CuEU",
#else
		"File browser",
#endif
		{
#if JAPANESE_UI
			HELP_ENABLE(FONT_UPARROW FONT_DOWNARROW, "J[\\ړ"),
			HELP_ENABLE(FONT_LEFTARROW FONT_RIGHTARROW, "J[\\1y[Wړ"),
			HELP_ENABLE(FONT_CIRCLE, "Q[N"),
			HELP_DISABLE(FONT_CROSS),
			HELP_ENABLE(FONT_SQUARE, "BIOSN"),
			HELP_ENABLE(FONT_TRIANGLE, "G~[^I"),
			HELP_ENABLE(FONT_LTRIGGER, "J[ݒ胁j[J"),
			HELP_ENABLE(FONT_RTRIGGER, "̃wvJ"),
			HELP_ENABLE("START", "XN[Vbgۑ"),
			HELP_ENABLE("SELECT", "I𒆂̃fBNgNfBNgɐݒ"),
#else
			HELP_ENABLE(FONT_UPARROW FONT_DOWNARROW, "Scroll"),
			HELP_ENABLE(FONT_LEFTARROW FONT_RIGHTARROW,"Scroll 1 page"),
			HELP_ENABLE(FONT_CIRCLE, "Launch game"),
			HELP_DISABLE(FONT_CROSS),
			HELP_ENABLE(FONT_SQUARE, "Boot BIOS"),
			HELP_ENABLE(FONT_TRIANGLE, "Exit emulator"),
			HELP_ENABLE(FONT_LTRIGGER, "Open color settings menu"),
			HELP_ENABLE(FONT_RTRIGGER, "Show this help"),
			HELP_ENABLE("START", "Save screen shot"),
			HELP_ENABLE("SELECT", "Set selected directory as start up"),
#endif
			HELP_EOM
		}
	},
	// HELP_GAMECONFIG
	{
#if JAPANESE_UI
		"Q[ݒ胁j[",
#else
		"Game configration menu",
#endif
		{
#if JAPANESE_UI
			HELP_ENABLE(FONT_UPARROW FONT_DOWNARROW, "J[\\ړ"),
			HELP_ENABLE(FONT_LEFTARROW FONT_RIGHTARROW, "lύX"),
			HELP_ENABLE(FONT_CIRCLE, "I"),
			HELP_ENABLE(FONT_CROSS, "Cj[ɖ߂"),
			HELP_DISABLE(FONT_SQUARE),
			HELP_DISABLE(FONT_TRIANGLE),
			HELP_DISABLE(FONT_LTRIGGER),
			HELP_ENABLE(FONT_RTRIGGER, "̃wvJ"),
			HELP_ENABLE("START", "XN[Vbgۑ"),
			HELP_DISABLE("SELECT"),
#else
			HELP_ENABLE(FONT_UPARROW FONT_DOWNARROW, "Scroll"),
			HELP_ENABLE(FONT_LEFTARROW FONT_RIGHTARROW,"Change value"),
			HELP_ENABLE(FONT_CIRCLE, "Select"),
			HELP_ENABLE(FONT_CROSS, "Return to main menu"),
			HELP_DISABLE(FONT_SQUARE),
			HELP_DISABLE(FONT_TRIANGLE),
			HELP_DISABLE(FONT_LTRIGGER),
			HELP_ENABLE(FONT_RTRIGGER, "Show this help"),
			HELP_ENABLE("START", "Save screen shot"),
			HELP_DISABLE("SELECT"),
#endif
			HELP_EOM
		}
	},
	// HELP_KEYCONFIG
	{
#if JAPANESE_UI
		"̓{^ݒ胁j[",
#else
		"Key configration menu",
#endif
		{
#if JAPANESE_UI
			HELP_ENABLE(FONT_UPARROW FONT_DOWNARROW, "J[\\ړ"),
			HELP_ENABLE(FONT_LEFTARROW FONT_RIGHTARROW, "lύX"),
			HELP_ENABLE(FONT_CIRCLE, "I"),
			HELP_ENABLE(FONT_CROSS, "Cj[ɖ߂"),
			HELP_DISABLE(FONT_SQUARE),
			HELP_DISABLE(FONT_TRIANGLE),
			HELP_DISABLE(FONT_LTRIGGER),
			HELP_ENABLE(FONT_RTRIGGER, "̃wvJ"),
			HELP_ENABLE("START", "XN[Vbgۑ"),
			HELP_DISABLE("SELECT"),
#else
			HELP_ENABLE(FONT_UPARROW FONT_DOWNARROW, "Scroll"),
			HELP_ENABLE(FONT_LEFTARROW FONT_RIGHTARROW,"Change value"),
			HELP_ENABLE(FONT_CIRCLE, "Select"),
			HELP_ENABLE(FONT_CROSS, "Return to main menu"),
			HELP_DISABLE(FONT_SQUARE),
			HELP_DISABLE(FONT_TRIANGLE),
			HELP_DISABLE(FONT_LTRIGGER),
			HELP_ENABLE(FONT_RTRIGGER, "Show this help"),
			HELP_ENABLE("START", "Save screen shot"),
			HELP_DISABLE("SELECT"),
#endif
			HELP_EOM
		}
	},
	// HELP_STATE
	{
#if JAPANESE_UI
		"Z[u/[hXe[g",
#else
		"Save/Load State",
#endif
		{
#if JAPANESE_UI
			HELP_ENABLE(FONT_UPARROW FONT_DOWNARROW, "J[\\ړ"),
			HELP_ENABLE(FONT_LEFTARROW FONT_RIGHTARROW, "ύX"),
			HELP_ENABLE(FONT_CIRCLE, "s"),
			HELP_DISABLE(FONT_CROSS),
			HELP_DISABLE(FONT_SQUARE),
			HELP_DISABLE(FONT_TRIANGLE),
			HELP_ENABLE(FONT_LTRIGGER, "Cj[ɖ߂"),
			HELP_ENABLE(FONT_RTRIGGER, "̃wvJ"),
			HELP_ENABLE("START", "XN[Vbgۑ"),
			HELP_DISABLE("SELECT"),
#else
			HELP_ENABLE(FONT_UPARROW FONT_DOWNARROW, "Scroll"),
			HELP_ENABLE(FONT_LEFTARROW FONT_RIGHTARROW,"Change function"),
			HELP_ENABLE(FONT_CIRCLE, "Select"),
			HELP_DISABLE(FONT_CROSS),
			HELP_DISABLE(FONT_SQUARE),
			HELP_DISABLE(FONT_TRIANGLE),
			HELP_ENABLE(FONT_LTRIGGER, "Return to main menu"),
			HELP_ENABLE(FONT_RTRIGGER, "Show this help"),
			HELP_ENABLE("START", "Save screen shot"),
			HELP_DISABLE("SELECT"),
#endif
			HELP_EOM
		}
	},
	// HELP_COLORSETTINGS
	{
#if JAPANESE_UI
		"J[ݒ胁j[",
#else
		"Color settings menu",
#endif
		{
#if JAPANESE_UI
			HELP_ENABLE(FONT_UPARROW FONT_DOWNARROW, "ڂ̑I"),
			HELP_ENABLE(FONT_LEFTARROW FONT_RIGHTARROW, "ڂ̑I/l̕ύX"),
			HELP_ENABLE(FONT_CIRCLE, "RGB̒l+5"),
			HELP_ENABLE(FONT_CROSS, "t@CuEUɖ߂"),
			HELP_ENABLE(FONT_SQUARE, "RGB̒l-5"),
			HELP_ENABLE(FONT_TRIANGLE, "SĂ̒lWɖ߂"),
			HELP_DISABLE(FONT_LTRIGGER),
			HELP_ENABLE(FONT_RTRIGGER, "̃wvJ"),
			HELP_ENABLE("START", "XN[Vbgۑ"),
			HELP_DISABLE("SELECT"),
#else
			HELP_ENABLE(FONT_UPARROW FONT_DOWNARROW, "Select item"),
			HELP_ENABLE(FONT_LEFTARROW FONT_RIGHTARROW,"Select item / Change value"),
			HELP_ENABLE(FONT_CIRCLE, "RGB value +5"),
			HELP_ENABLE(FONT_CROSS, "Return to file browser"),
			HELP_ENABLE(FONT_SQUARE, "RGB value -5"),
			HELP_ENABLE(FONT_TRIANGLE, "Restore all values to default"),
			HELP_DISABLE(FONT_LTRIGGER),
			HELP_ENABLE(FONT_RTRIGGER, "Show this help"),
			HELP_ENABLE("START", "Save screen shot"),
			HELP_DISABLE("SELECT"),
#endif
			HELP_EOM
		}
	},
	// HELP_COMMANDLIST
	{
#if JAPANESE_UI
		"R}hXg",
#else
		"Command list",
#endif
		{
#if JAPANESE_UI
			HELP_ENABLE(FONT_UPARROW FONT_DOWNARROW, "XgXN[/ڂI"),
			HELP_ENABLE(FONT_LEFTARROW FONT_RIGHTARROW, "1y[WXN["),
			HELP_ENABLE(FONT_CIRCLE, "ڃj[J/"),
			HELP_ENABLE(FONT_CROSS, "Q[/Cj[ɖ߂"),
			HELP_DISABLE(FONT_SQUARE),
			HELP_DISABLE(FONT_TRIANGLE),
			HELP_DISABLE(FONT_LTRIGGER),
			HELP_ENABLE(FONT_RTRIGGER, "̃wvJ"),
			HELP_ENABLE("START", "XN[Vbgۑ"),
			HELP_DISABLE("SELECT"),
#else
			HELP_ENABLE(FONT_UPARROW FONT_DOWNARROW, "Scroll/Select item"),
			HELP_ENABLE(FONT_LEFTARROW FONT_RIGHTARROW,"Scroll 1 page"),
			HELP_ENABLE(FONT_CIRCLE, "Open/Close item menu"),
			HELP_ENABLE(FONT_CROSS, "Return to main menu/game"),
			HELP_DISABLE(FONT_SQUARE),
			HELP_DISABLE(FONT_TRIANGLE),
			HELP_DISABLE(FONT_LTRIGGER),
			HELP_ENABLE(FONT_RTRIGGER, "Show this help"),
			HELP_ENABLE("START", "Save screen shot"),
			HELP_DISABLE("SELECT"),
#endif
			HELP_EOM
		}
	}
};


/*--------------------------------------------------------
	wv\
 -------------------------------------------------------*/

int help(int no)
{
	int i;
	char title[256];

	if (no > HELP_NUM_MAX) return 0;

	video_copy_rect32(show_frame, draw_frame, &full_rect, &full_rect);

	boxfill_alpha(0, 0, SCR_WIDTH - 1, SCR_HEIGHT - 1, COLOR_BLACK, 8);
	draw_dialog(59, 34, 419, 264);

#if JAPANESE_UI
	sprintf(title, "wv - %s", help_mes[no].menu_name);
#else
	sprintf(title, "Help - %s", help_mes[no].menu_name);
#endif
	uifont_print_shadow_center(43, UI_COLOR(UI_PAL_INFO), title);

	for (i = 0; help_mes[no].mes[i].text1[0]; i++)
	{
		int r = *help_mes[no].mes[i].r;
		int g = *help_mes[no].mes[i].g;
		int b = *help_mes[no].mes[i].b;

		uifont_print(73, 70 + (i << 4), r, g, b, help_mes[no].mes[i].text1);
		uifont_print(143, 70 + (i << 4), r, g, b, help_mes[no].mes[i].text2);
	}

#if JAPANESE_UI
	uifont_print_shadow_center(240, UI_COLOR(UI_PAL_SELECT), "{^ƃj[ɖ߂܂");
#else
	uifont_print_shadow_center(240, UI_COLOR(UI_PAL_SELECT), "Press any button to return to menu.");
#endif

	video_flip_screen(1);
	pad_wait_clear();
	pad_wait_press(-1);
	pad_wait_clear();
	video_flip_screen(1);

	return 0;
}
